#include <system.h> 
#include "ankpat.h"
#include "spi.h"

#define INVERSE_LCD

#define LCD_PORT portc
#define LCD_RST	3
#define LCD_SADATA	2
#define LCD_SCKL	1
#define LCD_CS	0

void LcdCls(void)
{
	int i,j;
	NSendCmd(0xae);
	for (i = 0; i < 132; i++){
		NSendCmd(0x75);   // page start/end ram
		NSendDat(i);            // for some reason starts at 2
		NSendDat(131);          
		NSendCmd(0x15);   // column start/end ram
		NSendDat(0);          
		NSendDat(131);
		NSendCmd(0x5c);    // write some stuff (background)
		for (j = 0; j < 66; j++){
			NSendDat(0xFF);  //white
			NSendDat(0xFF);
			NSendDat(0xFF);
		}
	}
	NSendCmd(0xaf);   // display on

}

void LCD_Init(void)
{
	//start LCD hardware-reset
	clear_bit(LCD_PORT, LCD_RST);
	delay_ms(200);
	//end LCD hardware-reset
	set_bit(LCD_PORT, LCD_RST);
	delay_ms(100);
	//Software Reset
	NSendCmd(0xca);

	//added
	NSendDat(0x03);//2division and Field
	NSendDat(32);//
	NSendDat(12);//
	NSendDat(0);

	NSendCmd(0xbb);  // comscn
	NSendDat(0x00);

	NSendCmd(0xd1);  // oscon
	delay_ms(100);
	
	NSendCmd(0x94);  // sleep out

	NSendCmd(0x81);  // electronic volume, this is kind a contrast/brightness
	NSendDat(0x1f);//ff);       //  this might be different for individual LCDs
	NSendDat(0x03);//01);     //

	NSendCmd(0x82);	//Temprature gradient
	NSendDat(0x00);	//default value
	
	
	NSendCmd(0x20);  // power ctrl
	NSendDat(0x0f);      //everything on, no external reference resistors
	delay_ms(100);
	
	NSendCmd(0xa6);// Normal display

	NSendCmd(0xa7);  // display mode

	NSendCmd(0xa9);// Partial area off

	NSendCmd(0x25);	//nop

	NSendCmd(0xbc);  // datctl
	NSendDat(0x00);
	NSendDat(0x00);
	NSendDat(0x02);

	NSendCmd(0x75);   // page start/end ram
	NSendDat(0);            // for some reason starts at 2
	NSendDat(131);          

	NSendCmd(0x15);   // column start/end ram
	NSendDat(0);          
	NSendDat(131);
	LcdCls();
	delay_ms(100);
	NSendCmd(0xaf);   // display on
}

void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y)
{
	x += 2;                  // for some reason starts at 2
	NSendCmd(0x75);   // page start/end ram
	NSendDat(x);            
	NSendDat(132);          
	NSendCmd(0x15);   // column start/end ram
	NSendDat(y+2);            // for some reason starts at 2
	NSendDat(131); 
	NSendCmd(0x5C);    // write some shit
	NSendDat(color); 
}

void setChr(char chr, char *adr)
{
	char i,j;
	char pos,idx;
	pos = chr;
	idx=pos*8;
	if(pos<0x20)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont0[i+idx];	
	}
	else if(pos<0x40)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont1[i+idx];	
	}
	else if(pos<0x60)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont2[i+idx];	
	}
	else if(pos<0x80)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont3[i+idx];	
	}
	else if(pos<0xA0)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont4[i+idx];	
	}
	else if(pos<0xC0)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont5[i+idx];	
	}
	else if(pos<0xE0)
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont6[i+idx];	
	}
	else
	{
		for(i=0;i<8;i++)
		*(adr+sizeof(char)*i)=AnkFont7[i+idx];	
	}

}

void DrawChr(char chr, char x, char y, short strcol,short bgcol)
{
	char i,j;
	short dot_e,dot_o;
	char dat[8];
	setChr(chr,dat);
	for(i=0;i<8;i++)
	{
		NSendCmd(0x75);//y set
		#ifdef INVERSE_LCD
		if(y>7)
		{
			NSendDat(195-(y*8+i));
		}
		else
		{
			NSendDat(y*8+i+2);
		}
		#else
		NSendDat(y*8+i+2);
		#endif
		NSendDat(131);
		NSendCmd(0x15);//x set
		NSendDat(x*6);
		NSendDat(131);
		NSendCmd(0x5C);
		for(j=0;j<3;j++)
		{
			if(dat[i]&(0x80>>(j*2)))
			{
				dot_e=strcol;
			}
			else
			{
				dot_e=bgcol;
			}
			
			if(dat[i]&(0x40>>(j*2)))
			{
				dot_o=strcol;
			}
			else
			{
				dot_o=bgcol;
			}
	
			NSendDat((0xFF&(char)(dot_e>>4)));
			NSendDat(0xF0&(char)(dot_e<<4) | 0x0F&(char)(dot_o>>8));
			NSendDat((char)(0xFF&dot_o));
		}
	
	}
}

void DrawStr(char *str, char x, char y, short strcol, short bgcol)
{
	char i;
	for(i=0;*(str+sizeof(char)*i)!='\0';i++)
		DrawChr(*(str+i),(x+i)%22,y+(x+i)/22,strcol,bgcol);
}